import os
from collections import defaultdict

import numpy as np


class KnowledgeGraph:
    def __init__(self, path, dataset):
        super(KnowledgeGraph, self).__init__()
        assert dataset in ['FB15k', 'FB15k-237', 'WN18', 'WN18RR']

        train_path = os.path.join(path, dataset, 'train2id.txt')
        valid_path = os.path.join(path, dataset, 'valid2id.txt')
        test_path = os.path.join(path, dataset, 'test2id.txt')
        entity_path = os.path.join(path, dataset, 'entity2id.txt')
        relation_path = os.path.join(path, dataset, 'relation2id.txt')

        self.entity2id = {}
        self.relation2id = {}
        with open(entity_path, "r", encoding='UTF-8') as f:
            for line in f.readlines():
                entity, id = line.strip().split("\t")
                self.entity2id[str(entity)] = int(id)
        with open(relation_path, "r", encoding='UTF-8') as f:
            for line in f.readlines():
                relation, id = line.strip().split("\t")
                self.relation2id[str(relation)] = int(id)
        self.num_entity = len(self.entity2id)
        self.num_relation = 2 * len(self.relation2id)

        self.train_data = self.read(train_path)
        self.valid_data = self.read(valid_path)
        self.test_data = self.read(test_path)
        self.data = np.concatenate((self.train_data, self.valid_data, self.test_data), axis=0)

        self.train_hr_vocab = self.get_hr_vocab(self.train_data)
        self.hr_vocab = self.get_hr_vocab(self.data)

    def read(self, file_path):
        with open(file_path, "r", encoding='UTF-8') as f:
            lines = f.readlines()
        triples = np.zeros((2 * len(lines), 3), dtype=np.int64)
        for i, line in enumerate(lines):
            h, r, t = line.strip().split("\t")
            triples[2 * i] = np.array([int(h), int(r), int(t)])
            triples[2 * i + 1] = np.array([int(t), int(r) + self.num_relation // 2, int(h)])
        return triples

    def get_hr_vocab(self, data):
        hr_vocab = defaultdict(list)
        for triple in data:
            hr_vocab[(triple[0], triple[1])].append(triple[2])
        return hr_vocab